// Aseprite Document Library
// Copyright (c) 2001-2016 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

#include "base/ints.h"

namespace doc {

  typedef uint32_t ObjectId;

  const ObjectId NullId = 0;

} // namespace doc
