// LibreSprite
// Copyright (C) 2021  LibreSprite contributors
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as
// published by the Free Software Foundation.

#pragma once

#include "ui/menu.h"

namespace app {

class ScriptMenu {
public:
  bool rebuildScriptsList(ui::Menu* menu);
};

}
