// Aseprite Render Library
// Copyright (c) 2001-2015 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

namespace render {

  enum class OnionskinPosition {
    BEHIND,
    INFRONT,
  };

} // namespace render
