// Aseprite UI Library
// Copyright (C) 2001-2015  David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

#include "she/keys.h"

namespace ui {

  typedef she::KeyModifiers KeyModifiers;
  typedef she::KeyScancode KeyScancode;

  using she::kKeyNoneModifier;
  using she::kKeyShiftModifier;
  using she::kKeyCtrlModifier;
  using she::kKeyAltModifier;
  using she::kKeyCmdModifier;
  using she::kKeySpaceModifier;
  using she::kKeyWinModifier;
  using she::kKeyUninitializedModifier;

  using she::kKeyNil;
  using she::kKeyA;
  using she::kKeyB;
  using she::kKeyC;
  using she::kKeyD;
  using she::kKeyE;
  using she::kKeyF;
  using she::kKeyG;
  using she::kKeyH;
  using she::kKeyI;
  using she::kKeyJ;
  using she::kKeyK;
  using she::kKeyL;
  using she::kKeyM;
  using she::kKeyN;
  using she::kKeyO;
  using she::kKeyP;
  using she::kKeyQ;
  using she::kKeyR;
  using she::kKeyS;
  using she::kKeyT;
  using she::kKeyU;
  using she::kKeyV;
  using she::kKeyW;
  using she::kKeyX;
  using she::kKeyY;
  using she::kKeyZ;
  using she::kKey0;
  using she::kKey1;
  using she::kKey2;
  using she::kKey3;
  using she::kKey4;
  using she::kKey5;
  using she::kKey6;
  using she::kKey7;
  using she::kKey8;
  using she::kKey9;
  using she::kKey0Pad;
  using she::kKey1Pad;
  using she::kKey2Pad;
  using she::kKey3Pad;
  using she::kKey4Pad;
  using she::kKey5Pad;
  using she::kKey6Pad;
  using she::kKey7Pad;
  using she::kKey8Pad;
  using she::kKey9Pad;
  using she::kKeyF1;
  using she::kKeyF2;
  using she::kKeyF3;
  using she::kKeyF4;
  using she::kKeyF5;
  using she::kKeyF6;
  using she::kKeyF7;
  using she::kKeyF8;
  using she::kKeyF9;
  using she::kKeyF10;
  using she::kKeyF11;
  using she::kKeyF12;
  using she::kKeyEsc;
  using she::kKeyTilde;
  using she::kKeyMinus;
  using she::kKeyEquals;
  using she::kKeyBackspace;
  using she::kKeyTab;
  using she::kKeyOpenbrace;
  using she::kKeyClosebrace;
  using she::kKeyEnter;
  using she::kKeyColon;
  using she::kKeyQuote;
  using she::kKeyBackslash;
  using she::kKeyBackslash2;
  using she::kKeyComma;
  using she::kKeyStop;
  using she::kKeySlash;
  using she::kKeySpace;
  using she::kKeyInsert;
  using she::kKeyDel;
  using she::kKeyHome;
  using she::kKeyEnd;
  using she::kKeyPageUp;
  using she::kKeyPageDown;
  using she::kKeyLeft;
  using she::kKeyRight;
  using she::kKeyUp;
  using she::kKeyDown;
  using she::kKeySlashPad;
  using she::kKeyAsterisk;
  using she::kKeyMinusPad;
  using she::kKeyPlusPad;
  using she::kKeyDelPad;
  using she::kKeyEnterPad;
  using she::kKeyPrtscr;
  using she::kKeyPause;
  using she::kKeyAbntC1;
  using she::kKeyYen;
  using she::kKeyKana;
  using she::kKeyConvert;
  using she::kKeyNoconvert;
  using she::kKeyAt;
  using she::kKeyCircumflex;
  using she::kKeyColon2;
  using she::kKeyKanji;
  using she::kKeyEqualsPad;
  using she::kKeyBackquote;
  using she::kKeySemicolon;
  using she::kKeyCommand;
  using she::kKeyUnknown1;
  using she::kKeyUnknown2;
  using she::kKeyUnknown3;
  using she::kKeyUnknown4;
  using she::kKeyUnknown5;
  using she::kKeyUnknown6;
  using she::kKeyUnknown7;
  using she::kKeyUnknown8;

  using she::kKeyFirstModifierScancode;

  using she::kKeyLShift;
  using she::kKeyRShift;
  using she::kKeyLControl;
  using she::kKeyRControl;
  using she::kKeyAlt;
  using she::kKeyAltGr;
  using she::kKeyLWin;
  using she::kKeyRWin;
  using she::kKeyMenu;
  using she::kKeyScrLock;
  using she::kKeyNumLock;
  using she::kKeyCapsLock;

  using she::kKeyScancodes;

} // namespace ui
