// Aseprite UI Library
// Copyright (C) 2016  David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

#include "she/pointer_type.h"

namespace ui {

  typedef she::PointerType PointerType;

} // namespace ui
