// Aseprite UI Library
// Copyright (C) 2001-2013  David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

namespace ui {

  enum MouseButtons {
    kButtonNone = 0,
    kButtonLeft = 1,
    kButtonRight = 2,
    kButtonMiddle = 4
  };

} // namespace ui
