// Aseprite Base Library
// Copyright (c) 2001-2016 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

#include "obs/connection.h"

namespace base {

using Connection = obs::connection;
using ScopedConnection = obs::scoped_connection;

} // namespace base
