// Observable Library
// Copyright (c) 2016 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#ifndef OBS_H_INCLUDED
#define OBS_H_INCLUDED
#pragma once

#include "obs/observable.h"
#include "obs/observers.h"
#include "obs/signal.h"
#include "obs/slot.h"

#endif
