// Aseprite Document Library
// Copyright (c) 2001-2014 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

namespace doc {

  typedef void (*AlgoHLine)(int x1, int y, int x2, void *data);

} // namespace doc
