// Aseprite Document Library
// Copyright (c) 2001-2015 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

#include "doc/brush.h"

#include <vector>

namespace doc {

  typedef std::vector<BrushRef> Brushes;

} // namespace doc
