// Aseprite Document Library
// Copyright (c) 2001-2014 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

namespace doc {

  // Dithering methods
  enum class DitheringMethod {
    NONE,
    ORDERED,
  };

} // namespace doc
