// Aseprite Base Library
// Copyright (c) 2015, 2016 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

#include "base/ints.h"
#include <vector>

namespace base {

typedef std::vector<uint8_t> buffer;

} // namespace base
