// Aseprite FreeType Wrapper
// Copyright (c) 2016 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

#define register
#include <ft2build.h>
#include FT_GLYPH_H
#include FT_FREETYPE_H
#undef register
