allegro_icon ICON data/icons/ase.ico
0 ICON data/icons/ase.ico
1 ICON data/icons/doc.ico

1 VERSIONINFO
  FILEVERSION 1,0,0,0
  PRODUCTVERSION 1,0,0,0
  FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
  FILEFLAGS 0x1L
#else
  FILEFLAGS 0x0L
#endif
  FILEOS 0x4L
  FILETYPE 0x1L
  FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "Comments", "https://github.com/LibreSprite/LibreSprite"
            VALUE "FileDescription", "LibreSprite - Animated sprites editor & pixel art tool"
            VALUE "FileVersion", "1,0,0,0"
            VALUE "InternalName", "libresprite"
            VALUE "LegalCopyright", "Copyright (C) 2001-2016 by David Capello, 2016-2023 LibreSprite contributors"
            VALUE "OriginalFilename", "libresprite.exe"
            VALUE "ProductName", "LIBRESPRITE"
            VALUE "ProductVersion", "1,1,0,0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END
