// Aseprite FLIC Library
// Copyright (c) 2015 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#ifndef FLIC_DETAILS_H_INCLUDED
#define FLIC_DETAILS_H_INCLUDED
#pragma once

#define FLI_MAGIC_NUMBER       0xAF11
#define FLC_MAGIC_NUMBER       0xAF12

#define FLI_FRAME_MAGIC_NUMBER 0xF1FA

#define FLI_COLOR_256_CHUNK    4
#define FLI_DELTA_CHUNK        7
#define FLI_COLOR_64_CHUNK     11
#define FLI_LC_CHUNK           12
#define FLI_BLACK_CHUNK        13
#define FLI_BRUN_CHUNK         15
#define FLI_COPY_CHUNK         16

#endif
