// Aseprite Document Library
// Copyright (c) 2001-2015 David Capello
//
// This file is released under the terms of the MIT license.
// Read LICENSE.txt for more information.

#pragma once

#include <memory>
#include "doc/image.h"

namespace doc {

  typedef std::shared_ptr<Image> ImageRef;

} // namespace doc
